local f = CreateFrame("Frame")
local strformat = string.format
local strgsub = string.gsub
local tinsert = table.insert
local hasGarrison, _ = C_Garrison.GetGarrisonInfo()
local NEW_IN_PROGRESS = "%d "..strgsub(GARRISON_LANDING_IN_PROGRESS, " %- %%d", "")
local NEW_AVAILABLE = "%d "..strgsub(GARRISON_LANDING_AVAILABLE, " %- %%d", "")
local NEW_WORK_COMPLETE = "    "..CAPACITANCE_WORK_COMPLETE_TOOLTIP_TITLE
local NEW_COMPLETE = "    "..GARRISON_LANDING_COMPLETED
local NEW_NEXT = "    "..GARRISON_LANDING_SHIPMENT_READY_TO_START
local NEW_MISSION_COMPLETE = GARRISON_MISSION_COMPLETE.."!"
local workOrderBuildings = {}
local missionData = {}
local underConstruction = {}

local GR = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Broker_GarrisonReport", {
	type = "data source",
	label = GARRISON_LANDING_PAGE_TITLE,
	icon = "Interface\\Icons\\Ability_Garrison_OrangeBird",
	text = "No Garrison",
	OnClick = GarrisonLandingPageMinimapButton_OnClick,
})

local function ScanBuildings()
	local buildings = C_Garrison.GetBuildings()
	for i = 1, #buildings do
		local bldg = buildings[i]
		local buildingID = bldg.buildingID
		workOrderBuildings[buildingID] = workOrderBuildings[buildingID] or {}
		workOrderBuildings[buildingID].plotID = bldg.plotID
		if C_Garrison.GetLandingPageShipmentInfo(buildingID) then
			workOrderBuildings[buildingID].canWork = true
		end
		
		local _, name, _, _, _, isBuilding, buildStart, buildDur, canActivate, _, _ = C_Garrison.GetOwnedBuildingInfoAbbrev(bldg.plotID)
		if isBuilding or canActivate then
			underConstruction[buildingID] = underConstruction[buildingID] or {}
			local newOrUpgrade = underConstruction[buildingID]
			newOrUpgrade.name = name
			newOrUpgrade.isBuilding = isBuilding
			newOrUpgrade.buildStart = buildStart
			newOrUpgrade.buildDur = buildDur
			newOrUpgrade.canActivate = canActivate
		end
	end
end

local function ShowConstruction(tt)
	if next(underConstruction) then	--if there are buildings under construction
		tt:AddLine(" ")
		tt:AddLine(GARRISON_LANDING_STATUS_BUILDING)
		for building, data in pairs(underConstruction) do
			tt:AddLine(data.name, 1, 1, 1)
			if data.isBuilding then
				tt:AddLine(strformat("    %s", GARRISON_BUILDING_IN_PROGRESS), 1, 1, 1)
			elseif data.canActivate then
				tt:AddLine(strformat("    %s", GARRISON_LANDING_BUILDING_COMPLEATE), 0, .95, 0)
			end
		end
	end
end

local function GetWorkOrders()
	C_Garrison.RequestLandingPageShipmentInfo()	--make sure we have the latest information
	
	for buildingID, data in pairs(workOrderBuildings) do
		if data.canWork then
			local name, texture, maxShipments, shipmentsReady, shipmentsTotal, 
				_, _, timeLeft, _, itemIcon = C_Garrison.GetLandingPageShipmentInfo(buildingID)
			
			data.name = name
			data.texture = texture
			data.max = maxShipments
			data.ready = shipmentsReady
			data.total = shipmentsTotal
			data.icon = itemIcon
			data.time = timeLeft
		end
	end
	
	--let's update our work order data once per minute
	--C_Timer.After(60, GetWorkOrders) --moved to ShowWorkOrders function
end

local function ShowWorkOrders(tt)
	GetWorkOrders()

	tt:AddLine(CAPACITANCE_WORK_ORDERS)
	for building, data in pairs(workOrderBuildings) do
		if data.canWork then
			tt:AddLine(data.name, 1, 1, 1)
			if data.total then	--if waiting on work orders
				if data.ready == data.total then	--all done!
					tt:AddLine(NEW_WORK_COMPLETE, 0, .95, 0)
				else
					if data.ready > 0 then	--if some are done
						tt:AddLine(strformat(NEW_COMPLETE, data.ready, data.total), .88, .84, .36)
					else	--if none are done yet
						tt:AddLine(strformat(NEW_COMPLETE, data.ready, data.total), 1, 1, 1)
					end
				end
			else	--no work orders made
				tt:AddLine(strformat(NEW_NEXT, data.max), 1, 1, 1)
			end
		end
	end
end

local function GetMissions()
	missionData.complete = #C_Garrison.GetCompleteMissions()
	missionData.onMission = #C_Garrison.GetInProgressMissions()	--all the missions we're working on
	missionData.inProgress = missionData.onMission - missionData.complete	--let's not include the complete # in our in progress #
	missionData.available = #C_Garrison.GetAvailableMissions()
	
	local textColor
	if missionData.onMission == 0 then
		textColor = "ffff0000"
	elseif missionData.complete == missionData.onMission then
		textColor = "ff00f200"
	else
		textColor = "ffffffff"
	end
	GR.text = strformat("|c%s%d/%d|r", textColor, missionData.complete, missionData.onMission)
end

local function ShowMissions(tt)
	tt:AddLine(GARRISON_MISSIONS)
	if missionData.inProgress == 0 then
		if missionData.onMission == 0 then
			tt:AddLine(GARRISON_EMPTY_IN_PROGRESS_LIST, 1, 0, 0)
		elseif missionData.complete == missionData.onMission then
			tt:AddLine(NEW_MISSION_COMPLETE, 0, .95, 0)
		end
		tt:AddLine(strformat(NEW_AVAILABLE, missionData.available), 1, 1, 1)
	else
		if missionData.complete > 0 then
			tt:AddLine(strformat(GARRISON_NUM_COMPLETED_MISSIONS, missionData.complete), 0, .95, 0)
		else
			tt:AddLine(strformat(GARRISON_NUM_COMPLETED_MISSIONS, missionData.complete), 1, 1, 1)
		end
		tt:AddLine(strformat(NEW_IN_PROGRESS, missionData.inProgress), 1, 1, 1)
	end
end

function GR.OnTooltipShow(self)
	self:AddLine("Broker_GarrisonReport",1,1,1)
	self:AddLine(" ")
	if hasGarrison then
		ShowMissions(self)
		self:AddLine(" ")
		ShowWorkOrders(self)
		ShowConstruction(self)
		self:AddLine(" ")
	end
	self:AddLine(MINIMAP_GARRISON_LANDING_PAGE_TOOLTIP)
end

f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("GARRISON_MISSION_FINISHED")
f:RegisterEvent("GARRISON_MISSION_LIST_UPDATE")
f:RegisterEvent("GARRISON_MISSION_NPC_OPENED")
f:RegisterEvent("GARRISON_BUILDING_PLACED")
f:RegisterEvent("GARRISON_BUILDING_REMOVED")
f:RegisterEvent("GARRISON_BUILDING_ACTIVATABLE")
f:RegisterEvent("GARRISON_BUILDING_ACTIVATED")
f:RegisterEvent("GARRISON_UPDATE")
f:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_LOGIN" then
		--initialize any sv's?
		GarrisonLandingPageMinimapButton:Hide()
		hooksecurefunc(GarrisonLandingPageMinimapButton, "Show", GarrisonLandingPageMinimapButton.Hide)
	elseif event == "GARRISON_UPDATE" then
		hasGarrison = C_Garrison.GetGarrisonInfo()
		GetMissions()
		ScanBuildings()
		GetWorkOrders()
	elseif event == "GARRISON_BUILDING_ACTIVATED" then
		local plotID, buildingID = ...
		underConstruction[buildingID] = nil
	elseif event == "GARRISON_MISSION_FINISHED" or event == "GARRISON_MISSION_LIST_UPDATE" or event == "GARRISON_MISSION_NPC_OPENED" then
		GetMissions()
	else
		ScanBuildings()
	end
end)